package	jp.co.sds_corp.hibiya.server;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;

/**
Networkの接続状況を監視します。
Socketで指定されたAddressのportに接続して状況を確認します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="Net1" class="jp.co.sds_corp.hibiya.server.NetChecker"&gt;
	&lt;Name&gt;192.168.0.1&lt;/Name&gt;
	&lt;Address&gt;192.168.0.1&lt;/Address&gt;
	&lt;Port&gt;22&lt;/Port&gt;
	&lt;ConnectionType&gt;TCP&lt;/ConnectionType&gt;
</PRE>

Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class NetChecker extends AbstractChecker
{
	private static final long serialVersionUID = 1L;
	String	address;
	int		port;
	String	connectionType;
	private int nTimeout = 10;

	public NetChecker() {
		setType( "Socket" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		setAddress( config.get( "Address", null ));
		setPort( Integer.valueOf( config.get( "Port", null )).intValue() );
		setConnectionType( config.get( "ConnectionType", null ));
		if ( getConnectionType() == null ) {
			setConnectionType( "TCP" );
		}
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			CmnDate startTime = new CmnDate();
			Socket	sock = new Socket( getAddress(), getPort() );
			sock.setSoTimeout( nTimeout * 1000 );
			InputStream	is = sock.getInputStream();
			BufferedReader rd = new BufferedReader( new InputStreamReader( is ));
			OutputStream	os = sock.getOutputStream();
			PrintWriter pw = new PrintWriter( os );
			pw.println( "" );
			if ( is.read() < 0 ) {
				throw( new Exception( "read error" ));
			}
			os.close();
			is.close();
			sock.close();
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			setCheckerStatus( IChecker.Status.OK );
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	/**
	* IPアドレス(address)を設定します。
	* @param	var		設定するデータ
	*/
	public void setAddress( String var ) {
		this.address = var ;
	}
	/**
	* IPアドレス(address)を取得します。
	* @return	addressの内容
	*/
	public String getAddress() {
		return address;
	}

	/**
	* ポート番号(port)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var ;
	}
	/**
	* ポート番号(port)を取得します。
	* @return	portの内容
	*/
	public int getPort() {
		return port;
	}

	/**
	* 接続タイプ(connectionType)を設定します。
	* @param	var		設定するデータ
	*/
	public void setConnectionType( String var ) {
		this.connectionType = var ;
	}
	/**
	* 接続タイプ(connectionType)を取得します。
	* @return	addressの内容
	*/
	public String getConnectionType() {
		return connectionType;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Address", address );
		config.addChild( "Port", String.valueOf( port ));
		config.addChild( "ConnectionType", connectionType );
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "Address=" + address + ";"
			+ "Port=" + port + ";"
			+ "ConnectionType=" + connectionType + ";"
			+ "]";
		return ret;
	}

}
