package	jp.co.sds_corp.hibiya.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
シェル(子プロセス)を起動して監視を実行します。
<BR>
XMLは以下の形式です。
<PRE>
&lt;Service name="Shell1" class="jp.co.sds_corp.hibiya.server.ShellChecker"&gt;
	&lt;Command&gt;check.sh&lt;/Command&gt;
	&lt;Env name="LANG" value="C"/&gt;
	&lt;Env name="LALL" value="C"/&gt;
	&lt;Dir&gt;/home&lt;/Dir&gt;
	&lt;NormalExit&gt;0&lt;/NormalExit&gt;
</PRE>
Env要素は複数指定することができます。Env要素とDir要素は省略可能です。

Action要素については、{@link jp.co.sds_corp.hibiya.action.IAction}
の説明を参照してください。
*/
public class ShellChecker extends AbstractChecker
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String	command;	// コマンドライン
	int	normalExit;	// 正常終了時のEXITコード
	String	dir;	// 実行時のディレクトリ
	HashMap<String,String> mapEnv = new HashMap<String,String>();	// 環境変数

	public ShellChecker() {
		setType( "Shell" );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		setCommand( config.get( "Command", null ));
		setNormalExit( config.getInteger( "NormalExit", 0 ));
		setDir( config.get( "Dir", null ));


		Iterator<IConfiguration> itr = config.getChilds( "Env" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String name = c.getAttributes().get( "name" );
			String value = c.getAttributes().get( "value" );
			mapEnv.put( name, value );
		}
	}

	/**
	* 監視処理を実行します。
	*/
	public void check() {
		try {
			CmnDate startTime = new CmnDate();
			execChild();
			CmnDate endTime = new CmnDate();
			clearResult();
			setResponseTime( startTime, endTime );

			setCheckerStatus( IChecker.Status.OK );
		} catch ( Throwable ex ) {
			synchronized( this ) {
				clearResult();
				setException( ex );
				setCheckerStatus( IChecker.Status.NG );
			}
		}
	}

	/**
	* 子プロセスを起動します。
	*/
	protected void execChild() throws Exception {
		if ( isDebug()) {
			logger.debug( this, "execute [" + command + "]" );
		}
		String[] aEnv;
		aEnv = new String[mapEnv.size()];
		int i = 0;
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			aEnv[i] = key + "=" + mapEnv.get( key );
			i++;
		}

		Process proc = Runtime.getRuntime().exec( command, aEnv, new File( dir ));
		InputReader or = new InputReader( getName(), "S", proc.getInputStream());
		Thread stdout = new Thread( or );
		stdout.start();
		InputReader er = new InputReader( getName(), "E", proc.getErrorStream());
		Thread stderr = new Thread( er );
		stderr.start();
		int nRet = proc.waitFor();
		if ( nRet != normalExit ) {
			throw( new Exception( "exit code=" + nRet ));
		}
	}

	private class InputReader implements Runnable
	{
		String  name;
		String  symbol;
		InputStream is;

		InputReader( String name, String symbol, InputStream is ) {
			this.name = name;
			this.symbol = symbol;
			this.is = is;
		}
		public void run() {
			try {
				BufferedReader rd = new BufferedReader(
										new InputStreamReader( is ));
				for ( ;; ) {
					String line = rd.readLine();
					if ( line == null ) {
						break;
					}
					// logger.debug( name + ":" + symbol + ": " + line );
				}
			} catch ( Exception ex ) {
				logger.error( this, "stream read error", ex );
			}
		}
	}

	/**
	* コマンドラインを設定します。
	* @param	var		設定するデータ
	*/
	public void setCommand( String var ) {
		this.command = var;
	}
	/**
	* コマンドラインを取得します。
	* @return	コマンドライン
	*/
	public String getCommand() {
		return command;
	}

	/**
	* 正常終了時のEXITコードを設定します。
	* @param	var		設定するデータ
	*/
	public void setNormalExit( int var ) {
		this.normalExit = var;
	}
	/**
	* 正常終了時のEXITコードを取得します。
	* @return	正常終了時のEXITコード
	*/
	public int getNormalExit() {
		return normalExit;
	}

	/**
	* 実行時のディレクトリを設定します。
	* @param	var		設定するデータ
	*/
	public void setDir( String var ) {
		this.dir = var;
	}
	/**
	* 実行時のディレクトリを取得します。
	* @return	実行時のディレクトリ
	*/
	public String getDir() {
		return dir;
	}

	/**
	* 環境変数を設定します。
	* @param	var		設定するデータ
	*/
	public void setEnv( HashMap<String,String> var ) {
		this.mapEnv = var;
	}
	/**
	* 環境変数を取得します。
	* @return	環境変数
	*/
	public HashMap<String,String> getEnv() {
		return mapEnv;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Command", command );
		config.addChild( "NormalExit", String.valueOf( normalExit ) );
		if ( dir != null ) {
			config.addChild( "Dir", dir );
		}
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			IConfiguration c = config.addChild( "Env" );
			c.setAttribute( "name", key );
			c.setAttribute( "value", (String )mapEnv.get( key ));
		}
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += "["
			+ "Command=" + command + ";"
			+ "NormalExit=" + normalExit + ";"
			+ "Dir=" + dir + ";"
			+ "Env=";
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			ret += key + "=" + mapEnv.get( key ) + ",";
		}
		ret += "]";
		return ret;
	}

}
